@@ -47,7 +47,7 @@ fn test01(db_name: &str, data: &[(String, String)]) -> std::io::Result<()> {
     _print_check_db_map(
         &db_map,
         CheckC {
-            check: true,
+            _check: true,
             ..Default::default()
         },
     );
@@ -57,7 +57,7 @@ fn test01(db_name: &str, data: &[(String, String)]) -> std::io::Result<()> {
 
 #[derive(Debug, Default, Clone, Copy)]
 struct CheckC {
-    check: bool,
+    _check: bool,
     f_mst: bool,
     f_graph: bool,
 }
